using System;
using System.Reflection;
using System.Collections;
using System.Collections.Specialized;

// Eine allgemeine Schnittstelle, die alle Plug-Ins implementieren mssen.
public interface IPlugin {
    void Start();
    void Stop();
}

// Eine einfache IPlugin-Implementierung, um den Einsatz der
// PluginManager-Kontrollklasse zu demonstrieren.
public class SimplesPlugin : IPlugin 
{

    public void Start() {
        Console.WriteLine(AppDomain.CurrentDomain.FriendlyName + 
            ": SimplesPlugin startet...");
    }

    public void Stop() {
        Console.WriteLine(AppDomain.CurrentDomain.FriendlyName + 
            ": SimplesPlugin stoppt...");
    }
}

// Die Kontrollklasse, die das Laden und Verndern
// von Plug-Ins innerhalb ihrer Anwendungsdomne verwaltet.
public class PluginManager : MarshalByRefObject {

    // Ein ListDictionary, um Schlsselreferenzen auf IPlugin-Instanzen zu speichern.
    private ListDictionary plugins = new ListDictionary();

    // Standardkonstruktor.
    public PluginManager() {}

    // Konstruktor, der whrend der Erstellungen bestimmte Plug-Ins ldt.
    public PluginManager(ListDictionary pluginList) {

        // Laden aller spezifizierten Plug-Ins.
        foreach (string plugin in pluginList.Keys) {

            this.LoadPlugin((string)pluginList[plugin], plugin);
        }
    }

    // Laden der spezifizierten Assembly und Instanziierung der spezifizierten
    // IPlugin-Implementierung aus dieser Assembly.
    public bool LadePlugin(string assemblyName, string pluginName) {

        try {

            // Laden der benannten privaten Assembly.
            Assembly assembly = Assembly.Load(assemblyName);

            // Erstellung der IPlugin-Instanz.
            IPlugin plugin = 
                (IPlugin)assembly.CreateInstance(pluginName, true);

            if (plugin != null) {

                // Neues IPlugin zu ListDictionary hinzufgen.
                plugins[pluginName] = plugin;

                return true;

            } else {
                return false;
            }
        } catch {
            return false;
        }
    }

    public void StartPlugin(string plugin) {

        // IPlugin aus dem ListDictionary extrahieren und Start aufrufen.
        ((IPlugin)plugins[plugin]).Start();
    }

    public void StopPlugin(string plugin) {

        // IPlugin aus dem ListDictionary extrahieren und Stop aufrufen.
        ((IPlugin)plugins[plugin]).Stop();
    }

    public ArrayList GetPluginList() {

		// Rckgabe einer Enumerationsliste von Plug-In-Namen. bernimmt die Schlssel
		// und ordnet sie in einer ArrayList an, welche das Wertbergabe-Marshaling
		// untersttzt.
        return new ArrayList(plugins.Keys);
    }
}

public class CreateInstanceBeispiel {

    public static void Main() {

        // Erstellen einer neuen Anwendungsdomne.
        AppDomain domain1 = AppDomain.CreateDomain("NeueAppDomain1");

        // Erstellen eines neuen PluginManagers in der neuen Anwendungsdomne 
        // durch den Standardkonstruktor.
        PluginManager manager1 = 
            (PluginManager)domain1.CreateInstanceAndUnwrap(
            "CreateInstanceBeispiel", "PluginManager");

        // Laden eines neuen Plug-Ins in die NeueAppDomain1.
        manager1.LadePlugin("CreateInstanceBeispiel", "SimplesPlugin");

        // Starten und stoppen des Plug-Ins in NeueAppDomain1.
        manager1.StartPlugin("SimplesPlugin");
        manager1.StopPlugin("SimplesPlugin");

        // Erstellen einer neuen Anwendungsdomne.
        AppDomain domain2 = AppDomain.CreateDomain("NeueAppDomain2");

        // Erstellen eines ListDictionary, das eine Liste aller zu erstellenden
    	// Plug-Ins enthlt.
        ListDictionary pluginList = new ListDictionary();
        pluginList["SimplesPlugin"] = "CreateInstanceBeispiel";

        // Erstellen eines neuen PluginManagers in der neuen Anwendungsdomne,
        // und spezifizieren einer Liste der standardmig zu erstellenden Plug-Ins.
        PluginManager manager2 = 
            (PluginManager)domain1.CreateInstanceAndUnwrap(
            "CreateInstanceBeispiel", "PluginManager", true, 0,
            null, new object[] {pluginList}, null, null, null);

        // Anzeige der Liste von Plug-Ins, die in NeueAppDomain2 geladen sind.
        Console.WriteLine("Plugins in NeueAppDomain2:");
        foreach (string s in manager2.GetPluginList()) {
            Console.WriteLine(" - " + s);
        }

        // Auf Tastendruck warten
        Console.ReadLine();
    }
}
