using System;
using System.Threading;
using System.Collections;

public class AsyncExecutionExample {

    // Ein Delegat, der Ihnen die asynchrone Ausfhrung von
	// AsyncExecutionExample.LongRunningMethod erlaubt.
	public delegate DateTime AsyncExampleDelegate(int delay, string name);

    // Eine eine lange Laufzeit simulierende Methode.
    public static DateTime LongRunningMethod(int delay, string name) {

        Console.WriteLine("{0} : {1} Beispiel - Thread wird gestartet.", 
            DateTime.Now.ToString("HH:mm:ss.ffff"), name);

        // Zeitintensiven Prozess simulieren.
        Thread.Sleep(delay);

        Console.WriteLine("{0} : {1} Beispiel - Thread wird beendet.", 
            DateTime.Now.ToString("HH:mm:ss.ffff"), name);

        // Den Beendigungszeitpunkt zurckliefern.
        return DateTime.Now;
    }

    // Diese Methode fhrt LongRunningMethod asynchron aus und fhrt mit weiteren
	// Verarbeitungen fort. Sobald die Verarbeitung abgeschlossen ist, blockiert die
	// Methode weitere Verarbeitungen solange bis LongRunningMethod beendet ist.
    public static void BlockingExample() {

        Console.WriteLine(Environment.NewLine + 
            "*** Blocken-Beispiel ausfhren ***");

        // LongRunningMethod asynchron ausfhren. Null sowohl fr den
		// Callback-Delegaten und das asynchrone Zustandsobjekt bergeben.
        AsyncExampleDelegate longRunningMethod = 
            new AsyncExampleDelegate(LongRunningMethod);

        IAsyncResult asyncResult = longRunningMethod.BeginInvoke(2000, 
            "Blocken", null, null);

        // Mit anderen Dingen fortfahren, bis zum Blocken bereit.
        for (int count = 0; count < 3; count++) {
            Console.WriteLine("{0} : Ausfhrung fortfahren bis zum Blocken bereit " 
                , DateTime.Now.ToString("HH:mm:ss.ffff"));
            Thread.Sleep(200);
        }

        // Blocken, bis die asynchrone Methode beendet wurde und die
		// Rckgabedaten ermitteln.
        Console.WriteLine("{0} : Blocken bis zum Methodenende...",
            DateTime.Now.ToString("HH:mm:ss.ffff"));
        DateTime completion = longRunningMethod.EndInvoke(asyncResult);

        // Ergebniswerte darstellen.
        Console.WriteLine("{0} : Blocken-Beispiel abgeschlossen.",
            completion.ToString("HH:mm:ss.ffff"));
    }

    // Diese Methode fhrt LongRunningMethod asynchron aus und dann
	// betritt sie eine Abfrage-(Polling-)Schleife, bis LongRunningMethod
	// abgeschlossen wurde.
    public static void PollingExample() {

        Console.WriteLine(Environment.NewLine + 
            "*** Abfrage-Beispiel ausfhren ***");

		// LongRunningMethod asynchron ausfhren. Null sowohl fr den
		// Callback-Delegaten und das asynchrone Zustandsobjekt bergeben.
		AsyncExampleDelegate longRunningMethod = 
            new AsyncExampleDelegate(LongRunningMethod);

        IAsyncResult asyncResult = longRunningMethod.BeginInvoke(2000, 
            "Abfragen", null, null);

        // Abfragen der asynchronen Methoden, um auf ihre Fertigstellung zu testen. 
		// Falls noch nicht beendet, Thread mit Sleep fr 300 ms vor dem nchsten Test aussetzen.
        Console.WriteLine("{0} : Wiederholtes Abfragen (Polling), bis Methode " +
            "beendet wurde...", DateTime.Now.ToString("HH:mm:ss.ffff"));
        while(!asyncResult.IsCompleted) {
            Console.WriteLine("{0} : Abfragen (Polling)...", 
                DateTime.Now.ToString("HH:mm:ss.ffff"));
            Thread.Sleep(300);
        }

        // Rckgabedaten fr die asynchrone Methode ermitteln.
        DateTime completion = longRunningMethod.EndInvoke(asyncResult);

        // Rckgabedaten darstellen.
        Console.WriteLine("{0} : Abfrage-(Polling-) Beispiel abgeschlossen.",
            completion.ToString("HH:mm:ss.ffff"));
    }

    // Diese Methode fhrt LongRunningMethod asynchron aus und verwendet
	// anschlieend ein WaitHandle um effizient auf die Fertigstellung
	// von LongRunningMethod zu warten. Die Verwendung eines Timeout erlaubt
	// der Methode, den Wartenprozess temporr zu verlassen, damit entweder
	// die Benutzeroberflche aktualisiert oder der gesamte Prozess wegen
	// zu langen Wartens komplett abgebrochen werden kann.
    public static void WaitingExample() {

        Console.WriteLine(Environment.NewLine + 
            "*** Ausfhren des Warten-Beispiels ***");

		// LongRunningMethod asynchron ausfhren. Null sowohl fr den
		// Callback-Delegaten und das asynchrone Zustandsobjekt bergeben.
		AsyncExampleDelegate longRunningMethod = 
            new AsyncExampleDelegate(LongRunningMethod);

        IAsyncResult asyncResult = longRunningMethod.BeginInvoke(2000, 
            "Waiting", null, null);

        // Auf die Fertigstellung der asynchronen Methode warten.
		// Timeout erfolgt nach 300 ms; dann die Status im Konsolenfenster
		// ausgeben und anschlieend wieder warten.
        Console.WriteLine("{0} : Warten bis zum Methodenende...", 
            DateTime.Now.ToString("HH:mm:ss.ffff"));
        while(!asyncResult.AsyncWaitHandle.WaitOne(300, false)) {
            Console.WriteLine("{0} : Warten-Timeout...",
                DateTime.Now.ToString("HH:mm:ss.ffff"));
        }

        // Rckgabedaten der asynchronen Methode ermitteln.
        DateTime completion = longRunningMethod.EndInvoke(asyncResult);

        // Rckgabedaten darstellen.
        Console.WriteLine("{0} : Warten-Beispiel abgeschlossen.",
            completion.ToString("HH:mm:ss.ffff"));
    }

    // Diese Methode fhrt LongRunningMethode asynchron mehrere Male aus
	// und verwendet anschlieend ein Array aus WaitHandle-Objekten,
	// um effizient auf die Fertigstellung aller Methoden zu warten.
	// Die Verwendung eines Timeout erlaubt
	// der Methode, den Wartenprozess temporr zu verlassen, damit entweder
	// die Benutzeroberflche aktualisiert oder der gesamte Prozess wegen
	// zu langen Wartens komplett abgebrochen werden kann.
    public static void WaitAllExample() 
	{

        Console.WriteLine(Environment.NewLine + 
            "*** Ausfhren des WaitAll-Beispiels ***");

        // Eine ArrayList, die die IAsyncResult-Instanzen fr jede gestartete
		// asynchrone Methode speichert.
        ArrayList asyncResults = new ArrayList(3);

        // Drei LongRunningMethods asynchron laufen lassen.
		// Null sowohl fr den Callback-Delegaten und das asynchrone Zustandsobjekt bergeben.
		// Die IAsyncResult-Instanz fr jede Methode zur ArrayList hinzufgen.
        AsyncExampleDelegate longRunningMethod = 
            new AsyncExampleDelegate(LongRunningMethod);

        asyncResults.Add(longRunningMethod.BeginInvoke(3000, 
            "WaitAll 1", null, null));

        asyncResults.Add(longRunningMethod.BeginInvoke(2500, 
            "WaitAll 2", null, null));

        asyncResults.Add(longRunningMethod.BeginInvoke(1500, 
            "WaitAll 3", null, null));

		// Ein Array aus WaitHandle-Objekten erstellen, die zum Warten auf
		// die Fertigstellung der asynchronen Methoden verwendet werden.
        WaitHandle[] waitHandles = new WaitHandle[3];

        for (int count = 0; count < 3; count++) {

            waitHandles[count] = 
                ((IAsyncResult)asyncResults[count]).AsyncWaitHandle;
        }

		// Auf die Fertigstellung aller drei asynchronen Methoden warten.
		// Timeout erfolgt nach 300 ms; dann die Status im Konsolenfenster
		// ausgeben und anschlieend wieder warten.
		Console.WriteLine("{0} : Warten bis alle 3 Methoden " + 
            "abgeschlossen sind...", DateTime.Now.ToString("HH:mm:ss.ffff"));
        while(!WaitHandle.WaitAll(waitHandles, 300, false)) {
            Console.WriteLine("{0} : WaitAll Timeout...", 
                DateTime.Now.ToString("HH:mm:ss.ffff"));
        }

        // Die Rckgabedaten jeder Methode untersuchen und feststellen,
		// wann jede Methode beendet wurde.
        DateTime completion = DateTime.MinValue;

        foreach (IAsyncResult result in asyncResults) {

            DateTime time = longRunningMethod.EndInvoke(result);
            if ( time > completion) completion = time;
        }

        // Rckgabewerte darstellen.
        Console.WriteLine("{0} : WaitAll-Beispiel abgeschlossen.",
            completion.ToString("HH:mm:ss.ffff"));
    }

    // Diese Methode fhrt LongRunningMethod asynchron aus und bergibt
	// eine AsyncCallback-Instanz. Die referenzierte CallbackHandler-Methode
	// wird automatisch aufgerufen, wenn die asynchrone Methode fertiggestellt wurde,
	// was dazu fhrt, dass diese Methode ohne Behinderung weiter ausgefhrt werden kann.
    public static void CallbackExample() {

        Console.WriteLine(Environment.NewLine + 
            "*** Ausfhren des Callback-Beispiels ***");

		// LongRunningMethod asynchron ausfhren. Eine AsyncCallback-Delegateninstanz
		// bergeben, die die CallbackHandler-Methode referenziert, die wiederum
		// automatisch aufgerufen wird, wenn die asynchrone Methode fertiggestellt wurde.
		// Eine Referenz auf die AsyncExampleDelegate-Delegateninstanz als asynchrones
		// Zustandsobjekt bergeben; anderenfalls hat die Callback-Methode keinen
		// Zugriff auf die Delegateninstanz, dessen EndInvoke-Methode sie aufrufen muss.
        AsyncExampleDelegate longRunningMethod = 
            new AsyncExampleDelegate(LongRunningMethod);

        IAsyncResult asyncResult = longRunningMethod.BeginInvoke(2000, 
            "Callback", new AsyncCallback(CallbackHandler), 
            longRunningMethod);

        // Mit anderen Arbeiten fortfahren.
        for (int count = 0; count < 15; count++) {
            Console.WriteLine("{0} : Mit weiterer Verarbeitung fortfahren...", 
                DateTime.Now.ToString("HH:mm:ss.ffff"));
            Thread.Sleep(200);
        }
    }

    // Eine Methode um den asynchronen Methodenabschluss durch Callbacks zu behandeln.
    public static void CallbackHandler(IAsyncResult result) {

        // Den Verweis zur AsyncExampleDelegate-Instanz aus der IAsyncResult-Instanz
		// extrahieren. Das erlaubt uns, die Rckgabedaten zu ermitteln.
        AsyncExampleDelegate longRunningMethod = 
            (AsyncExampleDelegate)result.AsyncState;

        // Rpckgabedaten der asynchronen Methode ermitteln.
        DateTime completion = longRunningMethod.EndInvoke(result);

        // Rckgabedaten darstellen.
        Console.WriteLine("{0} : Callback-Beispiel abgeschlossen.",
            completion.ToString("HH:mm:ss.ffff"));
    }

    public static void Main() {
        
        // Die verschiedenen Anstze demonstrieren:
        BlockingExample();
        PollingExample();
        WaitingExample();
        WaitAllExample();
        CallbackExample();

        // Warten, um fortzufahren.
        Console.WriteLine("");
        Console.WriteLine("Hauptmethode abgeschlossen. Drcken Sie Return.");
        Console.ReadLine();
    }
}