using System;
using System.IO;
using System.Threading;

public class AsynchronousIO
{
	public static void Main()
	{
		// Eine 1 MB groe Testdatei erstellen.
		FileStream fs = new FileStream("test.txt", FileMode.Create);
		fs.SetLength(1000000);
		fs.Close();

		// Die asynchrone Dateiverarbeitung in einem anderen Thread starten.
		AsyncProcessor asyncIO = new AsyncProcessor("test.txt");
		asyncIO.StartProcess();

		// Gleichzeitig eine andere Arbeit verrichten.
		// In diesem Beispiel fhren wir einfach 10 Sekunden lang eine Schleife aus.
		DateTime startTime = DateTime.Now;
		while (DateTime.Now.Subtract(startTime).TotalSeconds < 10)
		{
			Console.WriteLine("[HAUPT-THREAD]: Eine andere Arbeit verrichten.");

			// Pausieren, um eine zeitaufwndige Operation zu simulieren.
			Thread.Sleep(TimeSpan.FromMilliseconds(100));
		}
		
		Console.WriteLine("[HAUPT-THREAD]: Abgeschlossen.");
		Console.ReadLine();

		// Testdatei lschen.
		File.Delete("test.txt");
	}

}

public class AsyncProcessor
{
	private Stream inputStream;   

	// Die Menge, die in einem Block gelesen wird (2 KB).
	private int bufferSize = 2048;

	public int BufferSize
	{
		get {return bufferSize;}
		set {bufferSize = value;}
	}

	// Der Puffer, der die abgerufenen Daten aufnimmt.
	private byte[] buffer;

	public AsyncProcessor(string fileName)
	{
		buffer = new byte[bufferSize];

		// Datei asynchron (true) ffnen.
		inputStream = new FileStream(fileName, FileMode.Open,
			FileAccess.Read, FileShare.Read, bufferSize, true);
	}

	public void StartProcess()
	{
		// Den asynchronen Lesevorgang starten, um den Puffer zu fllen.
		inputStream.BeginRead(buffer, 0, buffer.Length,
			new AsyncCallback(OnCompletedRead), null);
	}

	private void OnCompletedRead(IAsyncResult asyncResult)
	{
		// Ein Block wurde asynchron gelesen.
		// Daten abrufen.
		int bytesRead = inputStream.EndRead(asyncResult);

		// Wenn keine Bytes gelesen wurden, ist der Stream am Ende der Datei.
		if (bytesRead > 0)
		{
			// Pausieren, um die Verarbeitung dieses Datenblocks zu simulieren.
			Console.WriteLine("\t[ASYNCHRONES LESEN]: Einen Block lesen.");
			Thread.Sleep(TimeSpan.FromMilliseconds(20));

			// Damit beginnen, den nchsten Block asynchron zu lesen.
			inputStream.BeginRead(
				buffer, 0, buffer.Length, new AsyncCallback(OnCompletedRead), null);
		}
		else
		{
			// Operation beenden.
			Console.WriteLine("\t[ASYNCHRONES LESEN]: Abgeschlossen.");
			inputStream.Close();
		}
	}
}