using System;
using System.Security;
using System.Security.Policy;
using System.Security.Cryptography.X509Certificates;

public class AppDomainPolicyExample {

    public static void Main() {
        
        // Eine neue Anwendungsdomne erstellen, in die die
        // heruntergeladenen Assemblies geladen werden.
        AppDomain domain = AppDomain.CreateDomain("modules");
        
        // Assemblies in die Anwendungsdomne laden, die durch
        // die Sicherheitsrichtlinie der Anwendungsdomne nicht
        // eingeschrnkt werden soll.
        // ...
        
        // Die Sicherheitsrichtlinie fr das neue
        // AppDomain-Objekt konfigurieren.
        SetDomainPolicy(domain);
        
        // Die heruntergeladenen Assemblies zur Ausfhrung in
        // die gesicherte Anwendungsdomne laden.
        // ...
    }

    // Eine Methode, die die Sicherheitsrichtlinie des
    // AppDomain-Objekts konfiguriert, das ihr bergeben
    // wurde. Die Sicherheitsrichtlinie weist jeder Assembly
    // abhngig von deren Publisher verschiedene Berechtigungen
    // zu. Assemblies von unbekannten Publishern erhalten keine
    // Berechtigungen.
    private static void SetDomainPolicy(AppDomain domain) {

        // Ein neues leeres PolicyLevel-Objekt fr die
        // Anwendungsdomne erstellen.
        PolicyLevel policy = PolicyLevel.CreateAppDomainLevel();
    
        // Ein neues FirstMatchCodeGroup-Objekt erstellen, das als
        // Stammknoten der Codegruppenhierarchie verwendet wird. Diese
        // Gruppe so konfigurieren, dass der gesamte Code betroffen ist.
        // Dazu wird die Mitgliedschaftsbedingung AllMembershipCondition
        // verwendet, und den Membern wird der benannte Berechtigungssatz
        // "Nothing" gewhrt. Dies bedeutet, dass alle Assemblies mit einem
        // leeren Berechtigungssatz fr die Sicherheitsrichtlinie der
        // Anwendungsdomne beginnen.
        policy.RootCodeGroup = new FirstMatchCodeGroup(
            new AllMembershipCondition(),
            new PolicyStatement(policy.GetNamedPermissionSet("Nothing"))
        );
            
        // Die Codegruppen erstellen, die die Berechtigungen bestimmen, die
        // einer Assembly zugewiesen werden, die von einem bestimmten
        // Publisher erstellt wurde. Da die Code-Stammgruppe ein
        // FirstMatchCodeGroup-Objekt ist, vergleicht die Richtlinienauflsung
        // die Assembly nur solange mit diesen untergeordneten Gruppen, bis
        // sie die erste bereinstimmung findet. Jede Codegruppe wird mit dem
        // Exclusive-Attribut erstellt, um sicherzustellen, dass die Assembly
        // keine zustzlichen Berechtigungen aus anderen Codegruppen verwendet.

        // Die Codegruppe erstellen, die den von Microsoft verffentlichten
        // Assemblies den Berechtigungssatz FullTrust gewhrt.
        X509Certificate microsoftCert = 
            X509Certificate.CreateFromCertFile("microsoft.cer");
        
        policy.RootCodeGroup.AddChild(new UnionCodeGroup(
            new PublisherMembershipCondition(microsoftCert),
            new PolicyStatement(policy.GetNamedPermissionSet("FullTrust"),
            PolicyStatementAttribute.Exclusive)
        ));

        // Die Codegruppe erstellen, die den von Litware, Inc. verffentlichten
        // Assemblies den Berechtigungssatz Internet gewhrt.
        X509Certificate litwareCert = 
            X509Certificate.CreateFromCertFile("litware.cer");
    
        policy.RootCodeGroup.AddChild(new UnionCodeGroup(
            new PublisherMembershipCondition(litwareCert),
            new PolicyStatement(policy.GetNamedPermissionSet("Internet"),
            PolicyStatementAttribute.Exclusive)
        ));

        // Die Codegruppe erstellen, die den von Fabrikam, Inc. verffentlichten
        // Assemblies den Berechtigungssatz Execution gewhrt.
        X509Certificate fabrikamCert = 
            X509Certificate.CreateFromCertFile("fabrikam.cer");
    
        policy.RootCodeGroup.AddChild(new UnionCodeGroup(
            new PublisherMembershipCondition(fabrikamCert),
            new PolicyStatement(policy.GetNamedPermissionSet("Execution"),
            PolicyStatementAttribute.Exclusive)
        ));

        // Eine letzte Codegruppe hinzufgen, um alle Assemblies abzufangen,
        // die keiner spezifischen Publisher-Gruppe zugeordnet werden knnen.
        // Dieser Gruppe wird der Berechtigungssatz Nothing zugewiesen. Da die
        // Gruppe mit dem Exclusive-Attribut ausgezeichnet ist, erhlt die
        // Assembly von anderen Gruppen und auch von hheren Richtlinienebenen
        // (Unternehmen, Computer und Benutzer) keine Berechtigungen.
        policy.RootCodeGroup.AddChild(new UnionCodeGroup(
            new AllMembershipCondition(),
            new PolicyStatement(policy.GetNamedPermissionSet("Nothing"),
            PolicyStatementAttribute.Exclusive)
        ));
 
        // Der zur Verfgung gestellten Anwendungsdomne die Richtlinie zuweisen.
        domain.SetAppDomainPolicy(policy);
    }
}
