using System;
using System.Threading;
using System.Runtime.InteropServices;
using System.Security;
using System.Security.Principal;
using System.Security.Permissions;

public class RoleBasedSecurityExample {
    
    public static void Main() {

		// Einen Verweis auf die aktuelle Anwendungsdomne abrufen.
		AppDomain appDomain = System.AppDomain.CurrentDomain;

		// Die aktuelle Anwendungsdomne so konfigurieren, das
		// Windows-Prinzipale verwendet werden.
		appDomain.SetPrincipalPolicy(
			System.Security.Principal.PrincipalPolicy.WindowsPrincipal);
        
        // Den Namen der Identitt vom aktuellen Prinzipal abrufen und anzeigen.
        // Dies veranlasst die Laufzeit auerdem, ein WindowsPrincipal-Objekt
        / fr die Anwendungsdomne zu instanziieren.
        Console.WriteLine(Thread.CurrentPrincipal.Identity.Name);

        // Versuchen, die verschiedenen statischen Methoden auszufhren. Zuerst
        // die 3 mit imperativen Anweisungen geschtzten Methoden, und dann die
        // 3 mit deklarativen Anweisungen geschtzten Methoden. Eine Meldung
        // anzeigen, wenn ein Fehler auftritt, was bedeutet, dass der aktuelle
        // Prinzipal die Sicherheitsforderung nicht erfllt.
        try {
            ImperativeExample.ProtectedMethod1();        
        } catch (SecurityException) {
            Console.WriteLine("SecurityException in ImperativeExample.ProtectedMethod1");
        }
        
        try {
            ImperativeExample.ProtectedMethod2();        
        } catch (SecurityException) {
            Console.WriteLine("SecurityException in ImperativeExample.ProtectedMethod2");
        }
        
        try {
            ImperativeExample.ProtectedMethod3();        
        } catch (SecurityException) {
            Console.WriteLine("SecurityException in ImperativeExample.ProtectedMethod3");
        }

        try {
            DeclarativeExample.ProtectedMethod1();        
        } catch (SecurityException) {
            Console.WriteLine("SecurityException in DeclarativeExample.ProtectedMethod1");
        }
        try {
            DeclarativeExample.ProtectedMethod2();        
        } catch (SecurityException) {
            Console.WriteLine("SecurityException in DeclarativeExample.ProtectedMethod2");
        }
        try {
            DeclarativeExample.ProtectedMethod3();        
        } catch (SecurityException) {
            Console.WriteLine("SecurityException in DeclarativeExample.ProtectedMethod3");
        }

        // Auf Tastendruck warten.
        Console.ReadLine();
    }
}

public class ImperativeExample {
    
	public static void ProtectedMethod1() { 
	    
		// Eine imperative rollenbasierte Sicherheitsforderung, die verlangt,
		// dass der aktuelle Prinzipal, eine Identitt namens "Anya"
		// reprsentiert. Die Rollen des Prinzipals sind hierbei nicht von
		// Bedeutung.
		System.Security.Permissions.PrincipalPermission perm = 
			new System.Security.Permissions.PrincipalPermission
			(@"MACHINE\Anya", null);
	        
		perm.Demand();
	}

	public static void ProtectedMethod2() { 

		// Eine imperative rollenbasierte Sicherheitsforderung, die verlangt,
		// dass der aktuelle Prinzipal ein Mitglied der Rollen "Managers" ODER
		// "Developers" ist. Wenn der Prinzipal ein Mitglied einer dieser Rollen
		// ist, wird der Zugriff gewhrt. Mit der Berechtigung
		// PrincipalPermission knnen Sie lediglich eine ODER-Typbeziehung (OR)
		// ausdrcken. Der Grund hierfr besteht darin, dass die Methode
		// PrincipalPolicy.Intersect solange eine leere Berechtigung zurckgibt,
		// bis die zwei Eingaben bereinstimmen. Sie knnen jedoch eine
		// Codelogik verwenden, um komplexere Bedingungen zu implementieren. In
		// diesem Fall ist der Name der Identitt nicht von Bedeutung.
		System.Security.Permissions.PrincipalPermission perm1 = 
			new System.Security.Permissions.PrincipalPermission
			(null, @"MACHINE\Managers");

		System.Security.Permissions.PrincipalPermission perm2 = 
			new System.Security.Permissions.PrincipalPermission
			(null, @"MACHINE\Developers");

		perm1.Union(perm2).Demand();
	}

	public static void ProtectedMethod3() { 

		// Eine imperative rollenbasierte Sicherheitsforderung, die verlangt,
		// dass der aktuelle Prinzipal eine Identitt namens "Anya"
		// reprsentiert UND (AND) ein Mitglied der Rolle "Managers" ist.
		System.Security.Permissions.PrincipalPermission perm = 
			new System.Security.Permissions.PrincipalPermission
			(@"MACHINE\Anya", @"MACHINE\Managers");
	        
		perm.Demand();
	}
}

public class DeclarativeExample {
    
    // Eine deklarative rollenbasierte Sicherheitsforderung, die verlangt,
    // dass der aktuelle Prinzipal, eine Identitt namens "Anya"
    // reprsentiert. Die Rollen des Prinzipals sind hierbei nicht von
    // Bedeutung.
    [PrincipalPermission(SecurityAction.Demand, Name = @"MACHINE\Anya")]
    public static void ProtectedMethod1() { /*...*/}

    // Eine deklarative rollenbasierte Sicherheitsforderung, die verlangt,
    // dass der aktuelle Prinzipal ein Mitglied der Rollen "Managers" ODER
    // "Developers" ist. Wenn der Prinzipal ein Mitglied einer dieser Rollen
    // ist, wird der Zugriff gewhrt. Sie knnen nur eine ODER-Typbeziehung
    // (OR) und keine UND-Beziehung (AND) ausdrcken. Der Name der Identitt
    // ist nicht von Bedeutung.
    [PrincipalPermission(SecurityAction.Demand, Role = @"MACHINE\Managers")]
    [PrincipalPermission(SecurityAction.Demand, Role = @"MACHINE\Developers")]    
    public static void ProtectedMethod2() { /*...*/}

    // Eine deklarative rollenbasierte Sicherheitsforderung, die verlangt,
    // dass der aktuelle Prinzipal eine Identitt namens "Anya"
    // reprsentiert UND (AND) ein Mitglied der Rolle "Managers" ist.
    [PrincipalPermission(SecurityAction.Demand, Name = @"MACHINE\Anya", 
        Role = @"MACHINE\Managers")]
    public static void ProtectedMethod3() { /*...*/}
}
