using System;
using System.IO;
using System.Security.Cryptography;

public class SymmetricEncryptionExample {

    public static void Main(string[] args) {
        
        // Einen neuen Triple DES-Algorithmus erstellen, um einen Schlssel abzurufen, der
        // im Beispiel verwendet wird. Dieser Schlssel wird von den Methoden 
        // EncryptFile und DecryptFile gemeinsam genutzt. Normalerweise wrde der Schlssel
        // vorab zwischen dem Absender und dem Empfnger vereinbart oder vom
        // Absender zusammen mit der verschlsselten Datei versendet. 
        // Dazu wrde ein Schlsselaustauschmechanismus verwendet.
        byte[] key;
        byte[] iv;
        
        using(SymmetricAlgorithm alg = SymmetricAlgorithm.Create("3DES")){
            
            key = alg.Key;
            iv = alg.IV;
        }
        
        // Datei verschlsseln. Das Prfix "encrypted" wird dem Namen der Quelldatei 
        // hinzugefgt und als Name fr die verschlsselte Datei verwendet. 
        // Einen Klon des Schlssels und IVs bergeben, da die Methode EncryptFile
        // diese Daten lscht.
        EncryptFile(args[0], "encrypted"+args[0], (byte[])key.Clone(), 
            (byte[])iv.Clone());
        
        // Verschlsselte Datei entschlsseln. Das Prfix "decrypted" wird dem Namen der Originaldatei 
        // hinzugefgt und als Name fr die entschlsselte Datei verwendet. 
        // Der Schlssel und IV mssen nicht geklont werden, obwohl
        // sie von DecryptFile gelscht werden. Sie werden jedoch nicht mehr bentigt.
        DecryptFile("encrypted"+args[0], "decrypted"+args[0], key, iv);
    }

    // Eine Methode, um eine angegebene Datei unter Verwendung des
    // zur Verfgung gestellten Schlssels und IVs mithilfe der Triple DES-Verschlsselung zu verschlsseln.
    private static void EncryptFile(string srcFileName, 
        string destFileName, byte[] key, byte[] iv) {
        
        // Streams erstellen, um auf die Quell- und Zieldateien zuzugreifen.
        Stream srcFile = 
          new FileStream(srcFileName, FileMode.Open, FileAccess.Read);
        Stream destFile = 
          new FileStream(destFileName, FileMode.Create, FileAccess.Write);
            
        // Einen neuen Triple DES-Algorithmus erstellen, um die Datei zu verschlsseln.
        using(SymmetricAlgorithm alg = SymmetricAlgorithm.Create("3DES")){

            // Die Eigenschaften Key und IV des symmetrischen Algorithmus basierend 
            // auf den zur Verfgung gestellten Werten konfigurieren.
            alg.Key = key;
            alg.IV = iv;

            // Einen CryptoStream erstellen, um die Inhalte des Quell-Streams zu verschlsseln,
            // wenn dieser gelesen wird. Die CreateEncryptor-Methode von 
            // SymmetricAlgorithm aufrufen, um eine verschlsselnde ICryptoTransform- 
            // Instanz zurckzugeben und CryptoStream zu bergeben.
            CryptoStream cryptoStream = new CryptoStream(srcFile, 
                                            alg.CreateEncryptor(),
                                            CryptoStreamMode.Read);

            // Einen Puffer deklarieren, der zum Lesen von Daten aus der Quell- 
            // datei ber CryptoStream und zum Schreiben der verschlsselten Datei verwendet werden soll.
            int bufferLength;
            byte[] buffer = new byte[1024];
            
            // Die Quelldatei in Blcken von 1024 Bytes lesen und die 
            // verschlsselte Version der Zieldatei schreiben.
            do {
                bufferLength = cryptoStream.Read(buffer, 0, 1024);
                destFile.Write(buffer, 0, bufferLength);
            } while (bufferLength > 0);
            
            // Die Stream-Ressourcen schlieen und die geheimen Daten aus 
            // den Objekten lschen, auf die wir nicht mehr verweisen werden.
            destFile.Flush();
            Array.Clear(key,0,key.Length);
            Array.Clear(iv,0,iv.Length);
            cryptoStream.Clear();                        
            cryptoStream.Close();
            srcFile.Close();
            destFile.Close();
        }
    }
    
    // Eine Methode, um eine angegebene Datei, die mit dem Triple DES-Verfahren verschlsselt wurde, mit dem 
    // zur Verfgung gestellten Schlssel und IV zu entschlsseln.
    private static void DecryptFile(string srcFileName, 
        string destFileName, byte[] key, byte[] iv) {

        // Streams erstellen, um auf die Quell- und Zieldateien zuzugreifen.
        Stream srcFile = 
          new FileStream(srcFileName, FileMode.Open, FileAccess.Read);
        Stream destFile = 
          new FileStream(destFileName, FileMode.Create, FileAccess.Write);
            
        // Einen neuen Triple DES-Algorithmus erstellen, um die Datei zu entschlsseln.
        using(SymmetricAlgorithm alg = SymmetricAlgorithm.Create("3DES")){

            // Die Eigenschaften Key und IV des symmetrischen Algorithmus basierend 
            // auf den zur Verfgung gestellten Werten konfigurieren.
            alg.Key = key;
            alg.IV = iv;
            
            // Einen CryptoStream erstellen, um die Inhalte der verschlsselten Daten 
            // zu entschlsseln, wenn diese geschrieben werden. Rufen Sie die CreateDecryptor-
            // Methode von SymmetricAlgorithm auf, um eine entschlsselnde
            // ICryptoTransform-Instanz zurckzugeben und CryptoStream zu bergeben.
            CryptoStream cryptoStream = new CryptoStream(destFile, 
                                        alg.CreateDecryptor(), 
                                        CryptoStreamMode.Write);

            // Einen Puffer deklarieren, der verwendet wird, um Daten aus der verschlsselten 
            // Datei zu lesen und ber CryptoStream in die entschlsselte Datei zu schreiben.
            int bufferLength;
            byte[] buffer = new byte[1024];
            
            // Die verschlsselte Datei in Blcken von 1024 Bytes lesen und 
            // die Klartextversion ber CryptoStream in die Zieldatei schreiben. 
            do {
                bufferLength = srcFile.Read(buffer, 0, 1024);
                cryptoStream.Write(buffer, 0, bufferLength);
            } while (bufferLength > 0);
            
            // Die Stream-Ressourcen schlieen und die geheimen Daten aus 
            // den Objekten lschen, auf die wir nicht mehr verweisen werden.
            cryptoStream.FlushFinalBlock();
            Array.Clear(key,0,key.Length);
            Array.Clear(iv,0,iv.Length);
            cryptoStream.Clear();                        
            cryptoStream.Close();
            srcFile.Close();
            destFile.Close();
        }
    }
}
