using System;
using System.Text;
using System.Security.Cryptography;

public class AsymmetricEncryptionExample {

    public static void Main(string[] args) {

        // Eine RSAParameters-Variable deklarieren, die die PUBLIC KEY-
        // Information fr den beabsichtigten Empfnger der verschlsselten
        // Nachricht enthlt.
        RSAParameters recipientsPublicKey;

        // Eine CspParameters-Variable deklarieren, die den Schlsselcontainer
        // identifiziert, in dem der private Schlssel (PRIVATE KEY) des
        // Empfngers gespeichert ist. In der Regel kann nur der Empfnger
        // auf diese Information zugreifen. Um mit dem Beispiel arbeiten zu
        // zu knnen, erstellen wir das RSA-Schlsselpaar zu Beginn des
        // Beispiels und ermglichen beiden Seiten (Empfnger und Sender)
        // den gemeinsamen Zugriff auf die Schlssel.
        CspParameters cspParams = new CspParameters();
        cspParams.KeyContainerName = "MyKeys";
        
        // Fr dieses Beispiel ein neues asymmetrisches Schlsselpaar unter
        // Verwendung der Klasse RSACryptoServiceProvider erstellen.
        // Die Schlssel in einem Schlsselspeicher namens "MyKeys" speichern
        // und den ffentlichen Schlssel in die Variable recipientsPublicKey 
        // extrahieren, sodass er im Beispiel verwendet werden kann.
        using (RSACryptoServiceProvider rsaAlg = 
            new RSACryptoServiceProvider(cspParams)) {

            // Den Algorithmus konfigurieren, um die Schlssel im Schlsselcontainer
            // zu speichern.
            rsaAlg.PersistKeyInCsp = true;
            
            // Den ffentlichen Schlssel extrahieren. Dies initiiert die
            // Erstellung der Schlssel 
            recipientsPublicKey = rsaAlg.ExportParameters(false);
        }
        
        // Die originale Klartextnachricht anzeigen.
        Console.WriteLine("Originalnachricht = {0}", args[0]);

        // Die Originalnachricht in ein Byte-Array konvertieren. Es ist besser,
        // geheime Informationen nicht als Zeichenfolgen zwischen den Methoden
        // zu bergeben.
        byte[] plaintext = Encoding.Unicode.GetBytes(args[0]);
                
        // Die Nachricht mit der EncryptMessage-Methode verschlsseln. Diese
        // Methode verlangt den ffentlichen Schlssel (PUBLIC KEY) der Person,
        // der wir die Nachricht senden mchten.
        byte[] ciphertext = EncryptMessage(plaintext, recipientsPublicKey);
        
        // Den von der EncryptMessage-Methode zurckgegebenen Chiffretext anzeigen.
        // Aus Grnden der Einfachheit wird die Methode BitConverter.ToString verwendet,
        // obwohl auf diese Weise Bindestriche (-) zwischen die Byte-Werte eingefgt
        // werden, was nicht der speicherinternen Darstellung der Daten entspricht.
        Console.WriteLine("Formatierter Chiffretext = {0}", 
            BitConverter.ToString(ciphertext));
        
        // Die verschlsselte Nachricht mit der Methode DecryptMessage entschlsseln.
        // Die DecryptMessage-Methode verlangt den Zugriff auf den privaten Schlssel (PRIVATE KEY)
        // des Empfngers, auf den normalerweise nur der Empfnger zugreifen kann.
        // Wir bergeben ein CspParameters-Objekt, das RSACryptoServiceProvider verrt,
        // in welchem Schlsselcontainer sich der private Schlssel (PRIVATE KEY)
        // befindet. Diese Lsung ist sicherer, als die direkte bergabe des
        // privaten Schlssels zwischen den Methoden.
        byte[] decData = DecryptMessage(ciphertext, cspParams);
        
        // Convert the decrypted message from a byte array to a string
        // and display it to the console.
        // Die verschlsselte Nachricht, die als Byte-Array vorliegt, in eine Zeichenfolge
        // konvertieren und im Konsolenfenster anzeigen.
        Console.WriteLine("Verschlsselte Nachricht = {0}", 
            Encoding.Unicode.GetString(decData));

        // Auf Tastendruck warten.
        Console.ReadLine();
    }

    // Eine Methode zur RSA-Verschlsselung einer Nachricht unter Verwendung
    // des ffentlichen Schlssels (PUBLIC KEY), der in einer RSAParameters-Struktur enthalten ist.
    private static byte[] EncryptMessage(byte[] plaintext, 
        RSAParameters rsaParams) {
        
        byte[] ciphertext = null;
        
        // Eine Instanz des RSA-Algorithmus erstellen.
        using (RSACryptoServiceProvider rsaAlg = 
            new RSACryptoServiceProvider()) {
        
            rsaAlg.ImportParameters(rsaParams);

            // Den Klartext mit der OAEP-Auffllung verschlsseln, die nur von
            // Windows XP und neueren Versionen untersttzt wird.
            ciphertext = rsaAlg.Encrypt(plaintext, true);
        }
        
        // Die Werte lschen, die im Byte-Array des Klartextes verwahrt werden. Dies
        // gewhrleistet, dass die geheimen Daten nicht im Speicher verbleiben, nachdem
        // Sie den Verweis darauf freigegeben haben.
        Array.Clear(plaintext, 0, plaintext.Length);
        
        return ciphertext;
    }

    // Eine Methode, um eine RSA-verschlsselte Nachricht zu entschlsseln. Dazu wird der
    // private Schlssel (PRIVATE KEY) verwendet, der sich in dem Container befindet,
    // der vom CspParameters-Objekt angegeben wird.
    private static byte[] DecryptMessage(byte[] ciphertext, 
        CspParameters cspParams ) {

        // Ein Byte-Array deklarieren, das den entschlsselten Klartext aufnimmt.
        byte[] plaintext = null;
        
        // Eine Instanz des RSA-Algorithmus erstellen.
        using (RSACryptoServiceProvider rsaAlg = 
            new RSACryptoServiceProvider(cspParams)) {

            // Den Klartext entschlsseln und die OAEP-Auffllung verwenden.
            plaintext = rsaAlg.Decrypt(ciphertext, true);
        }
        
        return plaintext;
    }
}
