using System;
using System.Text;
using System.Security.Cryptography;

public class KeyExchangeExample {

    public static void Main() {

        // Eine RSAParameters-Variable deklarieren, die den ffentlichen Schlssel
        // des beabsichtigten Empfngers des verschlsselten Sitzungsschlssels
        // aufnehmen wird.
        RSAParameters recipientsPublicKey;

        // Eine CspParameters-Variable deklarieren, die den Schlsselcontainer identifizieren
        // wird, in dem der private Schlssel des Empfngers gespeichert wird.
        // Normalerweise kann nur der Empfnger auf diese Information zugreifen.
        // Um mit dem Beispiel arbeiten zu knnen, erstellen wir das RSA-Schlsselpaar
        // zu Beginn des Beispiels und ermglichen beiden Seiten (Empfnger und Sender)
        // den gemeinsamen Zugriff auf die Schlssel.
        CspParameters cspParams = new CspParameters();
        cspParams.KeyContainerName = "MyKeys";
        
        // Fr dieses Beispiel ein neues asymmetrisches Schlsselpaar unter
        // Verwendung der Klasse RSACryptoServiceProvider erstellen.
        // Die Schlssel in einem Schlsselspeicher namens "MyKeys" speichern
        // und den ffentlichen Schlssel in die Variable recipientsPublicKey 
        // extrahieren, sodass er im Beispiel verwendet werden kann.
        using (RSACryptoServiceProvider rsaAlg = 
            new RSACryptoServiceProvider(cspParams)) {
            
            // Den Algorithmus konfigurieren, um die Schlssel im Schlsselcontainer
            // zu speichern.
            rsaAlg.PersistKeyInCsp = true;

            // Den ffentlichen Schlssel extrahieren. Dies initiiert die
            // Erstellung der Schlssel 
            recipientsPublicKey = rsaAlg.ExportParameters(false);
        }
        
        // Einen symmetrischen Triple-DES-Algorithmus erstellen und den automatisch
        // generierten Schlssel als Sitzungsschlssel verwenden.
        using (SymmetricAlgorithm symAlg =
            SymmetricAlgorithm.Create("3DES")) {
        
            // Den Quellsitzungsschlssel im Konsolenfenster anzeigen.
            Console.WriteLine("Situngsschlssel der Quelle = {0}\n\r", 
                BitConverter.ToString(symAlg.Key));
            
            // Den symmetrischen Sitzungsschlssel auf den Schlsselaustausch vorbereiten.
            // Dies geschieht mit der FormatKeyExchange-Methode, die sowohl den
            // zu verschlsselnden Schlssel als auch den ffentlichen Schlssel des
            // beabsichtigten Empfngers bentigt.
            byte[] exchangeData = 
                FormatKeyExchange(symAlg.Key, recipientsPublicKey);
            
            // Display the encrypted session key returned from the 
            // FormatKeyExchange method to the console.
            // Den von der Methode FormatKeyExchange zurckgegebenen verschlsselten
            // Sitzungsschlssel im Konsolenfenster anzeigen.
            Console.WriteLine("Austauschdaten = {0}\n\r", 
                BitConverter.ToString(exchangeData));

            // ****** Schlssel senden ******
            // Der verschlsselte Sitzungsschlssel kann nun ber die normalen
            // nicht verschlsselten Kommunikationskanle zum beabsichtigten
            // Empfnger gesendet werden.
            
            // Den Sitzungsschlssel aus den Austauschdaten extrahieren. Dies geschieht mit
            // der Methode DeformatKeyExchange. bergeben Sie die zuvor erstellte
            // Variable cspParams, die den privaten Schlssel des Empfngers zur Verfgung stellt.
            byte[] sessionKey = DeformatKeyExchange(exchangeData, cspParams);
            
            // Den extrahierten Sitzungsschlssel im Konsolenfenster anzeigen.
            Console.WriteLine("Sitzungsschlssel des Ziels = {0}\n\r", 
                BitConverter.ToString(sessionKey));

            // Auf Tastendruck warten.
            Console.ReadLine();
        }
    }

    // Methode zur Verschlsselung des symmetrischen Sitzungsschlssels, der fr den
    // Schlsselaustausch formatiert wird. Um den Sitzungsschlssel verschlsseln zu knnen,
    // mssen wir auf den ffentlichen RSA-Schlssel des beabsichtigten Empfngers zugreifen,
    // der in einer RSAParameters-Struktur an die Methode bergeben wird. Dieser Schlssel
    // htte uns zugesendet werden knnen, oder wir htten ihn von einem Schlssel-
    // weitergabedienst heruntergeladen.
    private static byte[] FormatKeyExchange(byte[] sessionKey, 
        RSAParameters rsaParams) {
        
        // Einen asymmetrischen RSA-Algorithmus erstellen.
        using (RSACryptoServiceProvider asymAlg = 
            new RSACryptoServiceProvider()) {
       
            // Den ffentlichen Schlssel des Empfngers in den Algorithmus importieren.
            asymAlg.ImportParameters(rsaParams);
            
            // Einen RSA-OAEP-Formatierer zum Formatieren der Schlsselaustauschdaten erstellen.
            RSAOAEPKeyExchangeFormatter formatter   
                = new RSAOAEPKeyExchangeFormatter();
    
            // Den RSA-Algorithmus fr die Verschlsselung des Sitzungsschlssels festlegen.
            formatter.SetKey(asymAlg);
    
            // Den Sitzungsschlssel verschlsseln und formatieren und das Ergebnis zurckgeben.
            return formatter.CreateKeyExchange(sessionKey);
        }
    }

    // Methode zur Entschlsselung der Schlsselaustauschdaten und zum Extrahieren
    // des symmetrischen Sitzungsschlssels. Um die Schlsselaustauschdaten zu entschlsseln,
    // mssen wir auf den privaten RSA-Schlssel zugreifen, den wir aus dem Schlsselcontainer
    // erhalten, der mit dem cspParams-Argument angegeben wird. Diese Vorgehensweise
    // gewhrleistet, dass wir geheime Daten nicht im Speicher verwahren und zwischen den
    // Methoden bergeben mssen.
    private static byte[] DeformatKeyExchange(byte[] exchangeData, 
        CspParameters cspParams) {   
        
        // Einen asymmetrischen RSA-Algorithmus erstellen.
       using (RSACryptoServiceProvider asymAlg = 
            new RSACryptoServiceProvider(cspParams)) {
        
            // Einen RSA-OAEP-Formatwiederhersteller zum Extrahieren des Sitzungsschlssels
            // aus den Schlsselaustauschdaten erstellen.
            RSAOAEPKeyExchangeDeformatter deformatter 
                = new RSAOAEPKeyExchangeDeformatter();
        
            // Specify the RSA algorithm as the one to use to decrypt the 
            // key exchange data.
            // Den RSA-Algorithmus fr die Entschlsselung der Schlsselaustauschdaten festlegen.
            deformatter.SetKey(asymAlg);

            // Die Schlsselaustauschdaten entschlsseln und den Sitzungsschlssel zurckgeben.
            return deformatter.DecryptKeyExchange(exchangeData);
        }
    }
}
