using System;
using System.IO;
using System.Text;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;

[Serializable]
public class Employee : ISerializable {

    private string name;
    private int age;
    private string address;
    
    // Einfacher Employee-Konstruktor
    public Employee(string name, int age, string address) {
    
        this.name = name;
        this.age = age;
        this.address = address;
    }            

    // Konstruktor, der es einem Formatter ermglicht, ein Employee-Objekt
    // zu deserialisieren. Sie sollten einen privaten oder zumindest einen geschtzten
    // Konstruktor deklarieren, um sicherzustellen, dass er nicht unntigerweise aufgerufen wird.
    private Employee(SerializationInfo info, StreamingContext context) {
        
        // Den Namen und das Alter des Angestellten extrahieren. Diese Daten werden unabhngig
        // vom Wert von StreamingContext immer in den serialisierten Daten prsent sein.
        name = info.GetString("Name");
        age = info.GetInt32("Age");
        
        // Versuchen, die Adresse des Angestellten zu extrahieren, und den entsprechenden
        // Fehler abfangen, wenn die Adresse nicht verfgbar ist.
        try { 
            address = info.GetString("Address");
        } catch (SerializationException) {
            address = null;
        }
    }

    // ffentliche Eigenschaft fr den Zugriff auf den Namen des Angestellten
    public string Name {
        get { return name; }
        set { name = value; }
    }

    // ffentliche Eigenschaft fr den Zugriff auf das Alter des Angestellten
    public int Age {
        get { return age; }
        set { age = value; }
    }
    
    // ffentliche Eigenschaft fr den Zugriff auf die Adresse des Angestellten.
    // Die verzgerte Initialisierung verwenden, um die Adresse einzurichten, da
    // ein deserialisiertes Objekt keinen Adresswert hat.    
    public string Address {
        get {
            if (address == null) {
                // Die Adresse von einem dauerhaften Speicher laden.
            }
            return address;
        }
        
        set {
            address = value;
        }
    }

    // Die von der Schnittstelle ISerializable deklarierte Methode GetObjectData
    // stellt den Mechanismus zur Verfgung, mit dem ein Formatter die Objektdaten
    // abruft, die serialisiert werden sollen.
    public void GetObjectData(SerializationInfo inf, StreamingContext con){
        
        // Den Namen und das Alter des Angestellten immer deserialisieren.
        inf.AddValue("Name", name);
        inf.AddValue("Age", age);

        // Die Adresse des Angestellten nicht serialisieren, wenn StreamingContext
        // anzeigt, dass die serialisierten Daten in eine Datei geschrieben werden
        if ((con.State & StreamingContextStates.File) == 0) {

            inf.AddValue("Address", address);
        }
    }
    
    // Object.ToString berschreiben, um eine Zeichenfolgendarstellung des
    // Angestelltenstatus zurckzugeben.
    public override string ToString() {
        
        StringBuilder str = new StringBuilder();
        
        str.AppendFormat("Name: {0}\n\r", Name);
        str.AppendFormat("Alter: {0}\n\r", Age);
        str.AppendFormat("Adresse: {0}\n\r", Address);
        
        return str.ToString();
    }
    
    public static void Main(string[] args) {
        
        // Ein Employee-Objekt erstellen, das Roger reprsentiert
        Employee roger = new Employee("Roger", 56, "London");
        
        // Roger anzeigen
        Console.WriteLine(roger);
        
        // Roger unter Angabe einer anderen Anwendungsdomne als Ziel der
        // serialisierten Daten serialisieren. Es werden alle Daten serialisiert.
        // dazu zhlt auch Rogers Adresse.
        Stream str = File.Create("roger.bin");
        BinaryFormatter bf = new BinaryFormatter();
        bf.Context = new StreamingContext(StreamingContextStates.CrossAppDomain);
        bf.Serialize(str, roger);
        str.Close();
        
        // Roger deserialisieren und anzeigen
        str = File.OpenRead("roger.bin");
        bf = new BinaryFormatter();
        roger = (Employee)bf.Deserialize(str);
        str.Close();
        Console.WriteLine(roger);        

        // Roger unter Angabe einer Datei als Ziel der serialisierten Daten
        // serialisieren. In diesem Fall wird Rogers Adresse nicht in
        // die serialisierten Daten eingebunden.
        str = File.Create("roger.bin");
        bf = new BinaryFormatter();
        bf.Context = new StreamingContext(StreamingContextStates.File);
        bf.Serialize(str, roger);
        str.Close();

        // Roger deserialisieren und anzeigen
        str = File.OpenRead("roger.bin");
        bf = new BinaryFormatter();
        roger = (Employee)bf.Deserialize(str);
        str.Close();
        Console.WriteLine(roger);        

		Console.ReadLine();
    }    
}
