using System;
using System.Collections;

public class Newspaper : IComparable {

    private string name;
    private int circulation;

    private class AscendingCirculationComparer : IComparer {
        
        int IComparer.Compare(object x, object y) {
            
            // Behanldungslogik fr null-Verweise, wie von der Schnittstelle
            // IComparer vorgeschrieben. Null gilt als kleinster Wert.
            if (x == null && y == null) return 0;
            else if (x == null) return -1;
            else if (y == null) return 1;
            
            // x und y verweisen auf dasselbe Objekt
            if (x == y) return 0;
            
            // berprfen, ob x und y Newspaper-Instanzen sind
            Newspaper newspaperX = x as Newspaper;
            if (newspaperX == null) {
                
                throw new ArgumentException("Ungltiger Objekttyp", "x");
            }
            
            Newspaper newspaperY = y as Newspaper;
            if (newspaperY == null) {
                
                throw new ArgumentException("Ungltiger Objekttyp", "y");
            }
            
            // Die verkauften Auflagen (circulation) miteinander vergleichen. IComparer schreibt vor, dass:
            //      ein Wert kleiner als 0 zurckgegeben werden muss, wenn x < y
            //      Null zurckgegeben werden muss, wenn x = y
            //      ein Wert grer als 0 zurckgegeben werden muss, wenn x > y
            // Diese Logik wird einfach mithilfe der Integer-Arithmetik implementiert.
            return newspaperX.circulation - newspaperY.circulation;
        }
    }
            
    public Newspaper(string name, int circulation) {
    
        this.name = name;
        this.circulation = circulation;
    }

    // Eine schreibgeschtzte Eigenschaft deklarieren, die eine Instanz von
    // AscendingCirculationComparer zurckgibt.
    public static IComparer CirculationSorter{
        get { return new AscendingCirculationComparer(); }
    }
    
    public override string ToString() {
        
        return string.Format("{0}: Circulation = {1}", name, circulation);
    }
    
    // Die Methode CompareTo vergleicht zwei Newspaper-Objekte, indem sie einen von der
    // Gro-/Kleinschreibung unabhnigen Vergleich der Zeitungsnamen durchfhrt.
    public int CompareTo(object obj) {
                
        // IComparable schreibt vor, dass ein Objekt immer grer als Null ist.
        if (obj == null) return 1;

        // 0 zurckgeben, wenn das andere Objekt ein Verweis auf dieses Objekt ist.
        if (obj == this) return 0;
                 
        // Versuchen, dass andere Objekt in eine Newspaper-Instanz zu konvertieren.
        Newspaper other = obj as Newspaper;
        
        // Wenn "other" null ist, muss es nicht in eine Newspaper-Instanz konvertiert werden.
        // IComparable schreibt vor, dass CompareTo in dieser Situation die Ausnahme
        // System.ArgumentException auslsen muss.
        if (other == null) {
            
            throw new ArgumentException("Ungltiger Objekttyp", "obj");
            
        } else {
            
            // Den Rckgabewert berechnen, indem ein von der Gro-/Kleinschreibung unabhngiger
            // Vergleich der Zeitungsnamen durchgefhrt wird.

            // Da der Zeitungsname eine Zeichenfolge ist, besteht die einfachste
            // Vorgehensweise darin, die Vergleichsmglichkeiten der Klasse String
            // zu nutzen, die whrend des Zeichenfolgenvergleichs auch die kulturellen
            // Einstellungen bercksichtigt.
            return string.Compare(this.name, other.name, true);
        }
    }
    
    public static void Main() {
        
        ArrayList newspapers = new ArrayList();
        
        newspapers.Add(new Newspaper("The Echo", 125780));
        newspapers.Add(new Newspaper("The Times", 55230));
        newspapers.Add(new Newspaper("The Gazette", 235950));
        newspapers.Add(new Newspaper("The Sun", 88760));
        newspapers.Add(new Newspaper("The Herald", 5670));
        
        Console.WriteLine("Nicht sortierte Zeitungsliste:");
        foreach (Newspaper n in newspapers) {
            
            Console.WriteLine(n);
        }
        
        Console.WriteLine(Environment.NewLine);
        Console.WriteLine("Nach dem Namen sortierte Zeitungsliste (Standardreihenfolge):");
        newspapers.Sort();
        foreach (Newspaper n in newspapers) {
            
            Console.WriteLine(n);
        }

        Console.WriteLine(Environment.NewLine);
        Console.WriteLine("Nach der Auflage sortierte Zeitungsliste:");
        newspapers.Sort(Newspaper.CirculationSorter);
        foreach (Newspaper n in newspapers) {
            
            Console.WriteLine(n);
        }

        Console.ReadLine();
    }
}
