using System;
using System.Collections;

// Die TeamMember-Klasse reprsentiert ein einzelnes Teammitglied.
public class TeamMember {
    
    public string Name;
    public string Title;

    // Einfacher TeamMember-Konstruktor.
    public TeamMember(string name, string title) {

        Name = name;
        Title = title;
    }
        
    // Eine Zeichenfolgendarstellung des TeamMember-Objekts zurckgeben.
    public override string ToString() {
        
        return string.Format("{0} ({1})", Name, Title);
    }        
}

// Die Team-Klasse reprsentiert eine Auflistung von TeamMember-Objekten. Sie implementiert
// die Schnittstelle IEnumerable, um die sequenzielle Verarbeitung von TeamMember-Objekten
// zu ermglichen.
public class Team : IEnumerable {

    // TeamMemberEnumerator ist eine private verschachtelte Klasse, die die Funktionalitt
      // zur Verfgung stellt, die zur sequenziellen Verarbeitung der TeamMember-Objekte in
      // einer Team-Auflistung notwendig ist. TeamMemberEnumerator kann wie eine verschachtelte
      // Klasse auf die privaten Member der Team-Klasse zugreifen.
    private class TeamMemberEnumerator : IEnumerator {

        // Das Team, das von diesem Objekt aufgelistet wird.
        private Team sourceTeam;

        // Boolescher Wert, der anzeigt, ob sich das zu Grunde liegende Team gendert hat,
            // was die Enumeration ungltig werden liee.
        private bool teamInvalid = false;

        // Integer, der das aktuelle TeamMember-Objekt identifiziert. Stellt den Index
            // des TeamMember-Objekts in der zu Grunde liegenden ArrayList zur Verfgung, der
            // von der Team-Auflistung verwendet wird. Dieser Index wird mit -1 initialisiert,
            // was dem Index vor dem ersten Element entspricht.
        private int currentMember = -1;

        // Der Konstruktor nimmt einen Verweis auf das Team entgegen, das die Quelle der
            // enumerierten Daten ist.
        internal TeamMemberEnumerator(Team team) {

            this.sourceTeam = team;

            // Bei sourceTeam registrieren, um nderungsbenachrichtigungen zu erhalten
            sourceTeam.TeamChange += 
                new TeamChangedEventHandler(this.TeamChange);
        }

        // Die Eigenschaft IEnumerator.Current implementieren.
        public object Current {
            get { 

                // Wenn TeamMemberEnumerator vor dem ersten oder hinter dem letzten
                        // Element positioniert wird, eine Ausnahme auslsen.
                if (currentMember == -1 ||
                    currentMember > (sourceTeam.teamMembers.Count-1)) {

                    throw new InvalidOperationException();
                }

                //Andernfalls das aktuelle TeamMember-Objekt zurckgeben
                return sourceTeam.teamMembers[currentMember];
            }
        }

        // Die Methode IEnumerator.MoveNext implementieren.
        public bool MoveNext() {

            // Wenn das zu Grunde liegende Team ungltig ist, Ausnahme auslsen
            if (teamInvalid) {
                
                throw new InvalidOperationException("Team wurde modifiziert");
            }
            
            // Andernfalls zum nchsten TeamMember-Objekt wechseln
            currentMember++;

            // false zurckgeben, wenn der Cursor hinter das letzte TeamMember-Objekt
                  // bewegt wurde
            if (currentMember > (sourceTeam.teamMembers.Count-1)) {
                return false;
            } else {
                return true;
            }
        }

        // Die Methode IEnumerator.Reset implementieren.
        // Diese Methode setzt die Position von TeamMemberEnumerator
            // auf den Anfang der TeamMembers-Auflistung zurck.
        public void Reset() {

            // Wenn das zu Grunde liegende Team ungltig ist, Ausnahme auslsen
            if (teamInvalid) {
                
                throw new InvalidOperationException("Team wurde modifiziert");
            }

            // Den aktuellen currentMember-Zeiger auf den Index setzen, der
                  // die Position vor dem ersten Element reprsentiert.
            currentMember = -1;
        }

            // Ein Ereignis-Handler, der auf die Nachricht reagiert, die mitteilt,
            // dass sich die zu Grunde liegende Team-Auflistung gendert hat.
        internal void TeamChange(Team t, EventArgs e) {
            
            // Signalisieren, dass das zu Grunde liegende Team nun ungltig ist
            teamInvalid = true;
        }
    }

      // Ein  Delegat der die Signatur festlegt, die alle Ereignis-Handler-Methoden
      // implementieren mssen, die auf Teamnderungen reagieren.
    public delegate void TeamChangedEventHandler(Team t, EventArgs e);

      // Eine  ArrayList fr die TeamMember-Objekte
    private ArrayList teamMembers;

      // Das Ereignis, das TeamMemberEnumerators darber benachrichtigt, dass
      // das Team gendert wurde.
    public event TeamChangedEventHandler TeamChange;

    // Team-Konstruktor
    public Team() {
        
        teamMembers = new ArrayList();
    }

    // Die Methode IEnumerable.GetEnumerator implementieren.
    public IEnumerator GetEnumerator() {
        return new TeamMemberEnumerator(this);
    }

    // Dem Team ein TeamMember-Objekt hinzufgen
    public void AddMember(TeamMember member) {
    
        teamMembers.Add(member);

        // Die Listener darber informieren, dass sich die Liste gendert hat
        if (TeamChange != null) {
            
            TeamChange(this, null);
        }
    }

    public static void Main() {

        // Ein neues Team erstellen
        Team team = new Team();
        team.AddMember(new TeamMember("Curly", "Clown"));
        team.AddMember(new TeamMember("Nick", "Knife Thrower"));
        team.AddMember(new TeamMember("Nancy", "Strong Man"));

        // Das Team sequenziell verarbeiten
        foreach (TeamMember member in team) {

            Console.WriteLine(member.ToString());
        }

        // Sequenzielle Verarbeitung mit einer While-Schleife
        IEnumerator e = team.GetEnumerator();
        while (e.MoveNext()) {

            Console.WriteLine(e.Current);
        }

            // Das Team sequenziell verarbeiten und versuchen, ein
            // Teammitglied hinzuzufgen. (Dies fhrt zur Auslsung
            // eine Ausnahme. )
        foreach (TeamMember member in team) {

            Console.WriteLine(member.ToString());
            team.AddMember(new TeamMember("Stumpy", "Lion Tamer"));
        }
    }
}
