using System;

// Die Schnittstelle IDisposable implementieren
public class DisposeExample : IDisposable {

    // Privater Datenmember, der anzeigt, ob die Objektressourcen
    // bereits freigegeben wurden.
    bool isDisposed = false;

    // Privater Datenmember, der ein Handle speichert, das auf eine
    // nicht verwaltete Ressource verweist.
    private IntPtr resourceHandle;

    // Konstruktor
    public DisposeExample() {

        // Konstruktorcode ruft Verweis auf nicht verwaltete Ressource ab.
        // resourceHandle = ...
    }

    // Destruktor / Finalisierer. Da Dispose GC.SuppressFinalize aufruft,
    // wird diese Methode nur dann von der Garbage Collection aufgerufen,
    // wenn der Benutzer des Objekts nicht wie vorgesehen Dispose aufruft.
    ~DisposeExample() {

        // Dispose-Methode aufrufen, anstatt den Code zu duplizieren, um
        // alle nicht verwalteten Ressourcen zu bereinigen. Die geschtzte
        // Dispose-berladung benutzen und den Wert "false" bergeben, um
        // anzuzeigen, dass Dispose whrend der Garbage Collection aufgerufen
        // wird und nicht vom Code des Objektbenutzers.
        Dispose(false);        
    }

    // ffentliche Implementierung der Methode IDisposable.Dispose, die vom
    // Benutzer des Objekts aufgerufen wird, um nicht verwaltete Ressourcen
    // deterministisch freizugeben.
    public void Dispose() {

        // Die geschtzte Dispose-berladung aufrufen und den Wert "true"
        // bergeben, um anzuzeigen, dass Dispose vom Code des Objektbenutzers
        // und nicht von der Garbage Collection aufgerufen wird.
        Dispose(true);

        // Da die Dispose-Methode die erforderliche Bereinigung durchfhrt,
        // muss sichergestellt werden, dass die Garbage Collection den
        // Klassendesktruktor nicht aufruft.
        GC.SuppressFinalize(this);
    }

    // Geschtzte berladung der Dispose-Methode. Das Argument disposing
    // signalisiert, ob die Methode vom Code des Objektbenutzers (true)
    // oder von der Garbage Collection (false) aufgerufen wird.
    protected virtual void Dispose(bool disposing) {

        // Versuchen Sie nicht, das Objekt zweimal zu bereinigen.
        if (!isDisposed) {

            // Ermitteln, ob der Code des Objektbenutzers oder die
            // Garbage Collection den Aufruf durchfhrt. Vermeiden Sie es,
            // andere Objekte whrend der Finalisierung aufzurufen.
            if (disposing) {

                // Methode wird vom Code des Objektbenutzers aufgerufen. Die
                // Dispose-Methode aller verwalteten Datenmember aufrufen, die
                // die Schnittstelle IDisposable implementieren.
                // ...
            }

            // Unabhngig davon, ob der Aufruf vom Code des Objektbenutzers oder
            // von der Garbage Collection erfolgt, mssen alle nicht verwalteten
            // Ressourcen freigegeben und die Werte der verwalteten Datenmember
            // auf null gesetzt werden.
            // Close(resourceHandle);
        }

        // Signalisieren, dass dieses Objekt bereinigt wurde.
        isDisposed = true;
    }

    // Vor der Ausfhrung von Funktionalitt sicherstellen, dass Dispose
    // nicht bereits fr das Objekt ausgefhrt wurde. 
    public void SomeMethod() {

        // Eine Ausnahme auslsen, wenn das Objekt bereits bereinigt wurde.
        if (isDisposed) {

            throw new ObjectDisposedException("DisposeExample");
        }

        // Methodenfunktionalitt ausfhren.
        // ...
    }

    public static void Main() {

        // Die Anweisung using gewhrleistet, dass die Dispose-Methode auch
        // auch dann aufgerufen wird, wenn eine Ausnahme eintritt.
        using (DisposeExample d = new DisposeExample()) {

            // Mit d arbeiten
        }
    }
}
