using System;
using System.Runtime.Serialization;

// CustomException als serialisierbar kennzeichnen. 
[Serializable]
public sealed class CustomException : ApplicationException {

    // Benutzerdefinierte Datenmember fr CustomException.
    private string stringInfo;
    private bool booleanInfo;

    // Drei Standardkonstruktoren. Einfach den Basisklassenkonstruktor
    // (System.ApplicationException) aufrufen.
    public CustomException() : base() {}

    public CustomException(string message): base(message) {}

    public CustomException(string message, Exception inner) 
        : base(message, inner) {}

    // Der von der Schnittstelle ISerialization bentigte Konstruktor.
    // Da CustomException versiegelt (sealed) ist, ist dieser
    // Konstruktor privat. Wre CustomException nicht versiegelt,
    // sollte dieser Konstruktor als geschtzt (protected) deklariert
    // werden, so dass ihn die abgeleiteten Klassen whrend der
    // Deserialisierung aufrufen knnten.
    private CustomException(SerializationInfo info, 
        StreamingContext context) : base (info, context) {

        // Jeden benutzerdefinierten Datenmember deserialisieren.
        stringInfo = info.GetString("StringInfo");
        booleanInfo = info.GetBoolean("BooleanInfo");
    }

    // Zustzliche Konstruktoren, die es dem Code ermglichen,
    // die benutzerdefinierten Datenmember zu setzen.
    public CustomException(string message, string stringInfo, 
        bool booleanInfo): this(message) {
    
        this.stringInfo = stringInfo;
        this.booleanInfo = booleanInfo;
    }

    public CustomException(string message, Exception inner, 
        string stringInfo, bool booleanInfo) : this(message, inner) {

        this.stringInfo = stringInfo;
        this.booleanInfo = booleanInfo;
    }

    // Schreibgeschtzte Eigenschaften, die den Zugriff auf die
    // benutzerdefinierten Datenmember ermglichen.
    public string StringInfo {
        get { return stringInfo; }
    }

    public bool BooleanInfo {
        get { return booleanInfo; }
    }

    // Die Methode GetObjectDatea (deklariert in der Schnittstelle ISerializable)
    // wird whrend der Serialisierung von CustomException verwendet.
    // Da CustomException benutzerdefinierte Datenmember deklariert,
    // muss es die Basisklassenimplementierung von GetObjectData
    // berschreiben.
    public override void GetObjectData(SerializationInfo info, 
        StreamingContext context) {

        // Die benutzerdefinierten Datenmember serialisieren.
        info.AddValue("StringInfo", stringInfo);
        info.AddValue("BooleanInfo", booleanInfo);

        // Basisklasse aufrufen, um ihre Member zu serialisieren.
        base.GetObjectData(info, context);
    }

    // Die Message-Eigenschaft der Basisklasse berschreiben, um die
    // benutzerdefinierten Datenmember einzubinden.
    public override string Message {
        get {
            string message = base.Message;
            if (stringInfo != null) {
                message += Environment.NewLine + 
                    stringInfo + " = " + booleanInfo;
            }
            return message;
        }
    }
    
    public static void Main () {

        // Ein CustomException-Objekt erstellen und auslsen.
        throw new CustomException("Ein Fehler", "SomeCustomMessage", true);
    }
}
