// Eine Ereignisargumentklasse, die Informationen ber ein
// Temperaturnderungsereignis enthlt. Eine Instanz dieser Klasse
// wird mit jedem Ereignis bergeben.
public class TemperatureChangeEventArgs : System.EventArgs {

    // Private Datenmember enthalten die alte und die neue Temperatur.
    private readonly int oldTemperature, newTemperature;
    
    // Konstruktor, der die alte und neue Temperatur entgegennimmt.
    public TemperatureChangeEventArgs(int oldTemp, int newTemp) {
    
        oldTemperature = oldTemp;
        newTemperature = newTemp;
    }
    
    // Schreibgeschtzte Eigenschaft fr den Zugriff auf die
    // Temperaturwerte.
    public int OldTemperature { get { return oldTemperature; } }
    public int NewTemperature { get { return newTemperature; } }
}
           
// Ein Delegat, der die Signatur festlegt, die alle
// Temperaturereignis-Handler-Methoden implementieren mssen.
public delegate void TemperatureEventHandler(Thermostat s, 
    TemperatureChangeEventArgs e);

// Ein Thermostat-Observer, der Informationen ber die
// Temperaturnderung anzeigt, wenn ein solches Ereignis
// eintritt.
public class TemperatureChangeObserver {
    
    // Ein Konstruktor, der einen Verweis auf das Thermostat-Objekt
    // entgegennimmt, das vom TemperatureChangeObserver-Objekt
    // observiert werden soll.
    public TemperatureChangeObserver(Thermostat t) {
        
        // Eine neue TemperatureEventHandler-Delegateninstanz erstellen
        // und beim angegebenen Thermostat-Objekt registrieren.
        t.TemperatureChange += 
            new TemperatureEventHandler(this.TemperatureChange);
    }
        
    // Die Methode zur Behandlung von Temperaturnderungsereignissen.
    public void TemperatureChange(Thermostat sender, 
        TemperatureChangeEventArgs temp) {
            
        System.Console.WriteLine
            ("ChangeObserver: Old={0}, New={1}, Change={2}",
            temp.OldTemperature, temp.NewTemperature, 
            temp.NewTemperature - temp.OldTemperature);
    }
}

// Ein Thermostat-Observer, der Informationen ber die durchschnittliche
// Temperatur anzeigt, wenn ein Temperaturnderungsereignis eintritt.
public class TemperatureAverageObserver {
    
    // sum enthlt die Summe aller Temperaturwerte.
    // count enthlt die Anzahl der empfangenen Temperaturereignisse.
    private int sum = 0, count = 0;
    
    // Ein Konstruktor, der einen Verweis auf das Thermostat-Objekt
    // entgegennimmt, das vom TemperatureAverageObserver-Objekt
    // observiert werden soll.
    public TemperatureAverageObserver (Thermostat t) {
        
        // Eine neue TemperatureEventHandler-Delegateninstanz erstellen
        // und beim angegebenen Thermostat registrieren.
        t.TemperatureChange += 
            new TemperatureEventHandler(this.TemperatureChange);
    }
        
    // Die Methode zur Behandlung von Temperaturnderungsereignissen.
    public void TemperatureChange(Thermostat sender, 
        TemperatureChangeEventArgs temp) {
            
        count++;
        sum += temp.NewTemperature;
            
        System.Console.WriteLine
            ("AverageObserver: Average={0:F}", (double)sum/(double)count);
    }
}

// Eine Klasse, die ein Thermostat reprsentiert, das als Quelle der
// Temperaturnderungsereignisse dient. Im Observer-Schema ist ein
// Thermostat-Objekt das "Subjekt", auf dessen nderungsbenachrichtigungen
// die Observer warten.
public class Thermostat {
    
    // Privates Feld fr die aktuelle Temperatur.
    private int temperature = 0;
    
    // Das Ereignis, das eine Liste der Observer-Delegaten verwaltet
    // und ein Temperaturnderungsereignis auslst, wenn sich eine
    // Temperatur ndert.
    public event TemperatureEventHandler TemperatureChange;

    // Eine geschtzte Methode zum Auslsen des Ereignisses TemperatureChange.
    // Da Ereignisse nur von dem Typ ausgelst werden knnen, in dem sie
    // enthalten sind, wird eine geschtzte Methode verwendet, um das
    // Ereignis auszulsen, sodass die abgeleiteten Klassen ein angepasstes
    // Verhalten zur Verfgung stellen und trotzdem auch weiterhin das
    // Basisklassenereignis auslsen knnen.
    virtual protected void RaiseTemperatureEvent
        (TemperatureChangeEventArgs e) {
        
        // Alle Observer benachrichtigen. Ein null-Test gibt Aufschluss
        // darber, ob die Observer registriert sind.
        if (TemperatureChange != null) {
            
            TemperatureChange(this, e);
        }
    }
        
    // ffentliche Eigenschaft zum Lesen und Schreiben der aktuellen
    // Temperatur. Die "set"-Seite der Eigenschaft ist fr die Auslsung
    // des Temperaturnderungsereignisses verantwortlich, das alle
    // Observer ber eine Temperaturnderung informiert.
    public int Temperature {
        
        get { return temperature; }
        
        set {
            // Ein neues Ereignisargumentobjekt erstellen, das die
            // alte und neue Temperatur enthlt.
            TemperatureChangeEventArgs e = 
                new TemperatureChangeEventArgs(temperature, value);
            
            // Aktuelle Temperatur aktualisieren.
            temperature = value;
            
            // Das Temperaturnderungsereignis auslsen.
            RaiseTemperatureEvent(e);
        }
    }
        
    public static void Main() {

        // Eine Thermostat-Instanz erstellen.
        Thermostat t = new Thermostat();
        
        // Die Thermostat-Observer erstellen.
        new TemperatureChangeObserver(t);
        new TemperatureAverageObserver(t);
        
        // In einer Schleife die Temperaturen vom Benutzer erfragen.
        // Jeder Wert, der kein Integer ist, beendet die Schleife.
        do {
            
            System.Console.Write("\n\rGeben Sie die aktuelle Temperatur ein: ");
            
            try {
                // Die Eingabe des Benutzers in einen Integer konvertieren
                // und verwenden, um die aktuelle Temperatur des Thermostats
                // zu setzen.
                t.Temperature = 
                    System.Int32.Parse(System.Console.ReadLine());
                
            } catch (System.Exception) {
                // Die Ausnahmebedingung benutzen, um das Programmende zu initiieren.
                System.Console.WriteLine("Terminating ObserverExample.");
                return;
            }
        } while (true);
    }
}
